@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2012-2015 Andrew M. Bishop
+ This file Copyright 2012-2015, 2017, 2019 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -203,6 +203,11 @@ static inline int32_t o5m_sint32(unsigned char **ptr)
 }
 
 
+#if defined(__clang__)
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunused-function"
+#endif
+
 /*++++++++++++++++++++++++++++++++++++++
   Parse an O5M int64 data value.
 
@@ -230,6 +235,10 @@ static inline int64_t o5m_int64(unsigned char **ptr)
  return(result);
 }
 
+#if defined(__clang__)
+#pragma clang diagnostic pop
+#endif
+
 
 /*++++++++++++++++++++++++++++++++++++++
   Parse an O5M sint64 data value.
@@ -541,7 +550,7 @@ static void process_way(void)
  int64_t delta_id;
  TagList *tags=NULL,*result=NULL;
  int mode=mode_change;
- unsigned char *refs=NULL,*refs_end;
+ unsigned char *refs=NULL,*refs_end=NULL;
 
  delta_id=o5m_sint64(&buffer_ptr);
  id+=delta_id;
@@ -620,7 +629,7 @@ static void process_relation()
  int64_t delta_id;
  TagList *tags=NULL,*result=NULL;
  int mode=mode_change;
- unsigned char *refs=NULL,*refs_end;
+ unsigned char *refs=NULL,*refs_end=NULL;
 
  delta_id=o5m_sint64(&buffer_ptr);
  id+=delta_id;
