@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2017 Andrew M. Bishop
+ This file Copyright 2008-2017, 2023 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -167,9 +167,9 @@ int main(int argc,char** argv)
  else if(option_filesort_ramsize==0)
    {
 #if SLIM
-    option_filesort_ramsize=64*1024*1024;
-#else
     option_filesort_ramsize=256*1024*1024;
+#else
+    option_filesort_ramsize=1024*1024*1024;
 #endif
    }
  else
@@ -357,17 +357,15 @@ if(!option_process_only)
 
  RemoveNonHighwayNodes(OSMNodes,OSMWays,option_keep||option_changes);
 
- /* Separate the segments and way names and sort them (must be before processing the segments) */
+ /* Separate the segments and way names and sort them (must be before indexing the segments) */
 
  OSMSegments=SplitWays(OSMWays,OSMNodes,option_keep||option_changes);
 
  SortWayNames(OSMWays);
 
- SortSegmentList(OSMSegments);
+ SortSegmentList(OSMSegments,OSMNodes,OSMWays);
 
- /* Process the segments and index them (must be before processing relations) */
-
- ProcessSegments(OSMSegments,OSMNodes,OSMWays);
+ /* Index the segments (must be before processing relations) */
 
  IndexSegments(OSMSegments,OSMNodes,OSMWays);
 
@@ -688,9 +686,9 @@ static void print_usage(int detail,const char *argerr,const char *err)
             "\n"
             "--sort-ram-size=<size>    The amount of RAM (in MB) to use for data sorting\n"
 #if SLIM
-            "                          (defaults to 64MB otherwise.)\n"
-#else
             "                          (defaults to 256MB otherwise.)\n"
+#else
+            "                          (defaults to 1024MB otherwise.)\n"
 #endif
 #if defined(USE_PTHREADS) && USE_PTHREADS
             "--sort-threads=<number>   The number of threads to use for data sorting.\n"
