@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2013 Andrew M. Bishop
+ This file Copyright 2008-2013, 2017 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -24,14 +24,15 @@
 #include <stdlib.h>
 
 #include "results.h"
+#include "logging.h"
 
 
 /*+ A queue of results. +*/
 struct _Queue
 {
- int      nincrement;           /*+ The amount to increment the queue when full. +*/
- int      nallocated;           /*+ The number of entries allocated. +*/
- int      noccupied;            /*+ The number of entries occupied. +*/
+ uint32_t nincrement;           /*+ The amount to increment the queue when full. +*/
+ uint32_t nallocated;           /*+ The number of entries allocated. +*/
+ uint32_t noccupied;            /*+ The number of entries occupied. +*/
 
  Result **results;              /*+ The queue of pointers to results. +*/
 };
@@ -58,6 +59,10 @@ Queue *NewQueueList(uint8_t log2bins)
 
  queue->results=(Result**)malloc(queue->nallocated*sizeof(Result*));
 
+#ifndef LIBROUTINO
+ log_malloc(queue->results,queue->nallocated*sizeof(Result*));
+#endif
+
  return(queue);
 }
 
@@ -82,6 +87,10 @@ void ResetQueueList(Queue *queue)
 
 void FreeQueueList(Queue *queue)
 {
+#ifndef LIBROUTINO
+ log_free(queue->results);
+#endif
+
  free(queue->results);
 
  free(queue);
@@ -103,7 +112,7 @@ void FreeQueueList(Queue *queue)
 
 void InsertInQueue(Queue *queue,Result *result,score_t score)
 {
- int index;
+ uint32_t index;
 
  if(result->queued==NOT_QUEUED)
    {
@@ -114,6 +123,10 @@ void InsertInQueue(Queue *queue,Result *result,score_t score)
       {
        queue->nallocated=queue->nallocated+queue->nincrement;
        queue->results=(Result**)realloc((void*)queue->results,queue->nallocated*sizeof(Result*));
+
+#ifndef LIBROUTINO
+       log_malloc(queue->results,queue->nallocated*sizeof(Result*));
+#endif
       }
 
     queue->results[index]=result;
@@ -128,7 +141,7 @@ void InsertInQueue(Queue *queue,Result *result,score_t score)
 
  while(index>1)
    {
-    int newindex;
+    uint32_t newindex;
     Result *temp;
 
     newindex=index/2;
@@ -161,7 +174,7 @@ void InsertInQueue(Queue *queue,Result *result,score_t score)
 
 Result *PopFromQueue(Queue *queue)
 {
- int index;
+ uint32_t index;
  Result *retval;
 
  if(queue->noccupied==0)
@@ -180,7 +193,7 @@ Result *PopFromQueue(Queue *queue)
 
  while((2*index)<queue->noccupied)
    {
-    int newindex;
+    uint32_t newindex;
     Result *temp;
 
     newindex=2*index;
@@ -203,7 +216,7 @@ Result *PopFromQueue(Queue *queue)
 
  if((2*index)==queue->noccupied)
    {
-    int newindex;
+    uint32_t newindex;
     Result *temp;
 
     newindex=2*index;
