@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2016 Andrew M. Bishop
+ This file Copyright 2008-2016, 2018 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -471,7 +471,7 @@ int main(int argc,char** argv)
 
        if(!translation)
          {
-          fprintf(stderr,"Warning: Cannot find a translation called '%s' in the file '%s'.\n",language,translations);
+          fprintf(stderr,"Error: Cannot find a translation called '%s' in the file '%s'.\n",language,translations);
           exit(EXIT_FAILURE);
          }
       }
@@ -481,7 +481,7 @@ int main(int argc,char** argv)
 
        if(!translation)
          {
-          fprintf(stderr,"Warning: No translations in '%s'.\n",translations);
+          fprintf(stderr,"Error: No translations in '%s'.\n",translations);
           exit(EXIT_FAILURE);
          }
       }
