@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2009-2015, 2017, 2019 Andrew M. Bishop
+ This file Copyright 2009-2015, 2017, 2019, 2023 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -64,10 +64,10 @@ typedef struct _thread_data
   void    **datap;              /*+ An array of pointers to the data objects. +*/
   size_t    n;                  /*+ The number of pointers. +*/
 
-  char    *filename;            /*+ The name of the file to write the results to. +*/
+  int       fd;                 /*+ The file descriptor of the file to write the results to. +*/
 
-  size_t   itemsize;            /*+ The size of each item. +*/
-  int    (*compare)(const void*,const void*); /*+ The comparison function. +*/
+  size_t    itemsize;           /*+ The size of each item. +*/
+  int     (*compare)(const void*,const void*); /*+ The comparison function. +*/
  }
  thread_data;
 
@@ -76,7 +76,6 @@ typedef struct _thread_data
 #if defined(USE_PTHREADS) && USE_PTHREADS
 
 static pthread_mutex_t running_mutex = PTHREAD_MUTEX_INITIALIZER;
-static pthread_mutex_t files_mutex   = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t  running_cond  = PTHREAD_COND_INITIALIZER;
 
 #endif
@@ -131,6 +130,7 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
  void **datap;
  thread_data *threads;
  int i,more=1;
+ char *filename=(char*)malloc_logassert(strlen(option_tmpdirname)+24);
 #if defined(USE_PTHREADS) && USE_PTHREADS
  int nthreads=0;
 #endif
@@ -142,7 +142,10 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
  if(nitems==0)
     return(0);
 
- if((nitems*(itemsize+sizeof(void*)))<option_filesort_ramsize)
+ if((nitems*(itemsize+sizeof(void*)))<(option_filesort_ramsize/option_filesort_threads))
+    /* use one thread */
+    nitems=nitems+1;
+ else if((nitems*(itemsize+sizeof(void*)))<option_filesort_ramsize)
     nitems=1+nitems/option_filesort_threads;
  else
     nitems=option_filesort_ramsize/(option_filesort_threads*(itemsize+sizeof(void*)));
@@ -151,14 +154,14 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
 
  for(i=0;i<option_filesort_threads;i++)
    {
+    threads[i].fd=-1;
+
     threads[i].data=malloc_logassert(nitems*itemsize);
     threads[i].datap=malloc_logassert(nitems*sizeof(void*));
 
     log_malloc(threads[i].data ,nitems*itemsize);
     log_malloc(threads[i].datap,nitems*sizeof(void*));
 
-    threads[i].filename=(char*)malloc_logassert(strlen(option_tmpdirname)+24);
-
     threads[i].itemsize=itemsize;
     threads[i].compare=compare_function;
    }
@@ -173,7 +176,31 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
 
     if(option_filesort_threads>1)
       {
-       /* Find a spare slot (one *must* be unused at all times) */
+       /* If all threads are in use wait for an existing thread to finish */
+
+       if(nthreads==option_filesort_threads)
+         {
+          pthread_mutex_lock(&running_mutex);
+
+          while(nthreads==option_filesort_threads)
+            {
+             for(i=0;i<option_filesort_threads;i++)
+                if(threads[i].running==2)
+                  {
+                   pthread_join(threads[i].thread,NULL);
+                   threads[i].running=0;
+                   CloseFileBuffered(threads[i].fd);
+                   nthreads--;
+                  }
+
+             if(nthreads==option_filesort_threads)
+                pthread_cond_wait(&running_cond,&running_mutex);
+            }
+
+          pthread_mutex_unlock(&running_mutex);
+         }
+
+       /* Find a spare slot */
 
        pthread_mutex_lock(&running_mutex);
 
@@ -219,56 +246,45 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
     if(threads[thread].n==0)
        break;
 
-    /* Sort the data pointers using a heap sort (potentially in a thread) */
+    /* Shortcut if only one file, don't write to disk */
 
-    sprintf(threads[thread].filename,"%s/filesort.%d.tmp",option_tmpdirname,nfiles);
+    if(more==0 && nfiles==0)
+       filesort_heapsort(threads[thread].datap,threads[thread].n,threads[thread].compare);
+
+    else
+      {
+       /* Create the file descriptor (not thread-safe) */
+
+       sprintf(filename,"%s/filesort.%d.tmp",option_tmpdirname,nfiles);
+
+       threads[thread].fd=OpenFileBufferedNew(filename);
+
+       if(option_filesort_threads==1)
+         {
+          filesort_fixed_heapsort_thread(&threads[thread]);
+
+          CloseFileBuffered(threads[thread].fd);
+         }
 
 #if defined(USE_PTHREADS) && USE_PTHREADS
 
-    /* Shortcut if only one file, don't write to disk */
-
-    if(more==0 && nfiles==0)
-       filesort_heapsort(threads[thread].datap,threads[thread].n,threads[thread].compare);
-    else if(option_filesort_threads>1)
-      {
-       pthread_mutex_lock(&running_mutex);
-
-       while(nthreads==(option_filesort_threads-1))
+       else
          {
-          for(i=0;i<option_filesort_threads;i++)
-             if(threads[i].running==2)
-               {
-                pthread_join(threads[i].thread,NULL);
-                threads[i].running=0;
-                nthreads--;
-               }
+          pthread_mutex_lock(&running_mutex);
 
-          if(nthreads==(option_filesort_threads-1))
-             pthread_cond_wait(&running_cond,&running_mutex);
+          threads[thread].running=1;
+
+          pthread_mutex_unlock(&running_mutex);
+
+          pthread_create(&threads[thread].thread,NULL,(void* (*)(void*))filesort_fixed_heapsort_thread,&threads[thread]);
+
+          nthreads++;
          }
 
-       threads[thread].running=1;
-
-       pthread_mutex_unlock(&running_mutex);
-
-       pthread_create(&threads[thread].thread,NULL,(void* (*)(void*))filesort_fixed_heapsort_thread,&threads[thread]);
-
-       nthreads++;
-      }
-    else
-       filesort_fixed_heapsort_thread(&threads[thread]);
-
-#else
-
-    /* Shortcut if only one file, don't write to disk */
-
-    if(more==0 && nfiles==0)
-       filesort_heapsort(threads[thread].datap,threads[thread].n,threads[thread].compare);
-    else
-       filesort_fixed_heapsort_thread(&threads[thread]);
-
 #endif
 
+      }
+
     nfiles++;
    }
  while(more);
@@ -288,6 +304,7 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
          {
           pthread_join(threads[i].thread,NULL);
           threads[i].running=0;
+          CloseFileBuffered(threads[i].fd);
           nthreads--;
          }
 
@@ -314,7 +331,7 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
          }
       }
 
-    DeleteFile(threads[0].filename);
+    DeleteFile(filename);
 
     goto tidy_and_exit;
    }
@@ -329,8 +346,6 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
 
  for(i=0;i<nfiles;i++)
    {
-    char *filename=threads[0].filename;
-
     sprintf(filename,"%s/filesort.%d.tmp",option_tmpdirname,i);
 
     fds[i]=ReOpenFileBuffered(filename);
@@ -461,12 +476,12 @@ index_t filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*pre_sort_funct
 
     free(threads[i].data);
     free(threads[i].datap);
-
-    free(threads[i].filename);
    }
 
  free(threads);
 
+ free(filename);
+
  return(count_out);
 }
 
@@ -514,6 +529,7 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
  void **datap;
  thread_data *threads;
  int i,more=1;
+ char *filename=(char*)malloc_logassert(strlen(option_tmpdirname)+24);
 #if defined(USE_PTHREADS) && USE_PTHREADS
  int nthreads=0;
 #endif
@@ -529,7 +545,10 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
     one will require RAM for data, FILESORT_VARALIGN and sizeof(void*)
     Assume that data+FILESORT_VARALIGN+sizeof(void*) is 4*data. */
 
- if((datasize*4)<option_filesort_ramsize)
+ if((datasize*4)<option_filesort_ramsize/option_filesort_threads)
+    /* use one thread */
+    datasize=datasize*4;
+ else if((datasize*4)<option_filesort_ramsize)
     datasize=(datasize*4)/option_filesort_threads;
  else
     datasize=option_filesort_ramsize/option_filesort_threads;
@@ -540,13 +559,13 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
 
  for(i=0;i<option_filesort_threads;i++)
    {
+    threads[i].fd=-1;
+
     threads[i].data=malloc_logassert(datasize);
     threads[i].datap=NULL;
 
     log_malloc(threads[i].data,datasize);
 
-    threads[i].filename=(char*)malloc_logassert(strlen(option_tmpdirname)+24);
-
     threads[i].compare=compare_function;
    }
 
@@ -564,7 +583,31 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
 
     if(option_filesort_threads>1)
       {
-       /* Find a spare slot (one *must* be unused at all times) */
+       /* If all threads are in use wait for an existing thread to finish */
+
+       if(nthreads==option_filesort_threads)
+         {
+          pthread_mutex_lock(&running_mutex);
+
+          while(nthreads==option_filesort_threads)
+            {
+             for(i=0;i<option_filesort_threads;i++)
+                if(threads[i].running==2)
+                  {
+                   pthread_join(threads[i].thread,NULL);
+                   threads[i].running=0;
+                   CloseFileBuffered(threads[i].fd);
+                   nthreads--;
+                  }
+
+             if(nthreads==option_filesort_threads)
+                pthread_cond_wait(&running_cond,&running_mutex);
+            }
+
+          pthread_mutex_unlock(&running_mutex);
+         }
+
+       /* Find a spare slot */
 
        pthread_mutex_lock(&running_mutex);
 
@@ -625,59 +668,45 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
     if(threads[thread].n==0)
        break;
 
-    /* Sort the data pointers using a heap sort (potentially in a thread) */
+    /* Shortcut if only one file, don't write to disk */
 
     if(more==0 && nfiles==0)
-       threads[thread].filename[0]=0;
+       filesort_heapsort(threads[thread].datap,threads[thread].n,threads[thread].compare);
+
     else
-       sprintf(threads[thread].filename,"%s/filesort.%d.tmp",option_tmpdirname,nfiles);
+      {
+       /* Create the file descriptor (not thread-safe) */
+
+       sprintf(filename,"%s/filesort.%d.tmp",option_tmpdirname,nfiles);
+
+       threads[thread].fd=OpenFileBufferedNew(filename);
+
+       if(option_filesort_threads==1)
+         {
+          filesort_vary_heapsort_thread(&threads[thread]);
+
+          CloseFileBuffered(threads[thread].fd);
+         }
 
 #if defined(USE_PTHREADS) && USE_PTHREADS
 
-    /* Shortcut if only one file, don't write to disk */
-
-    if(more==0 && nfiles==0)
-       filesort_heapsort(threads[thread].datap,threads[thread].n,threads[thread].compare);
-    else if(option_filesort_threads>1)
-      {
-       pthread_mutex_lock(&running_mutex);
-
-       while(nthreads==(option_filesort_threads-1))
+       else
          {
-          for(i=0;i<option_filesort_threads;i++)
-             if(threads[i].running==2)
-               {
-                pthread_join(threads[i].thread,NULL);
-                threads[i].running=0;
-                nthreads--;
-               }
+          pthread_mutex_lock(&running_mutex);
 
-          if(nthreads==(option_filesort_threads-1))
-             pthread_cond_wait(&running_cond,&running_mutex);
+          threads[thread].running=1;
+
+          pthread_mutex_unlock(&running_mutex);
+
+          pthread_create(&threads[thread].thread,NULL,(void* (*)(void*))filesort_vary_heapsort_thread,&threads[thread]);
+
+          nthreads++;
          }
 
-       threads[thread].running=1;
-
-       pthread_mutex_unlock(&running_mutex);
-
-       pthread_create(&threads[thread].thread,NULL,(void* (*)(void*))filesort_vary_heapsort_thread,&threads[thread]);
-
-       nthreads++;
-      }
-    else
-       filesort_vary_heapsort_thread(&threads[thread]);
-
-#else
-
-    /* Shortcut if only one file, don't write to disk */
-
-    if(more==0 && nfiles==0)
-       filesort_heapsort(threads[thread].datap,threads[thread].n,threads[thread].compare);
-    else
-       filesort_vary_heapsort_thread(&threads[thread]);
-
 #endif
 
+      }
+
     nfiles++;
    }
  while(more);
@@ -697,6 +726,7 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
          {
           pthread_join(threads[i].thread,NULL);
           threads[i].running=0;
+          CloseFileBuffered(threads[i].fd);
           nthreads--;
          }
 
@@ -725,7 +755,7 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
          }
       }
 
-    DeleteFile(threads[0].filename);
+    DeleteFile(filename);
 
     goto tidy_and_exit;
    }
@@ -742,8 +772,6 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
 
  for(i=0;i<nfiles;i++)
    {
-    char *filename=threads[0].filename;
-
     sprintf(filename,"%s/filesort.%d.tmp",option_tmpdirname,i);
 
     fds[i]=ReOpenFileBuffered(filename);
@@ -886,12 +914,12 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
     log_free(threads[i].data);
 
     free(threads[i].data);
-
-    free(threads[i].filename);
    }
 
  free(threads);
 
+ free(filename);
+
  return(count_out);
 }
 
@@ -906,35 +934,22 @@ index_t filesort_vary(int fd_in,int fd_out,int (*pre_sort_function)(void*,index_
 
 static void *filesort_fixed_heapsort_thread(thread_data *thread)
 {
- int fd;
  size_t item;
 
  /* Sort the data pointers using a heap sort */
 
  filesort_heapsort(thread->datap,thread->n,thread->compare);
 
- /* Create a temporary file and write the result */
+ /* Write the result to the given temporary file */
 
-#if defined(USE_PTHREADS) && USE_PTHREADS
-
- if(option_filesort_threads>1)
-    pthread_mutex_lock(&files_mutex);
-
-#endif
-
- fd=OpenFileBufferedNew(thread->filename);
-
- for(item=0;item<thread->n;item++)
-    WriteFileBuffered(fd,thread->datap[item],thread->itemsize);
-
- CloseFileBuffered(fd);
+ if(thread->fd > 0)
+    for(item=0;item<thread->n;item++)
+       WriteFileBuffered(thread->fd,thread->datap[item],thread->itemsize);
 
 #if defined(USE_PTHREADS) && USE_PTHREADS
 
  if(option_filesort_threads>1)
    {
-    pthread_mutex_unlock(&files_mutex);
-
     pthread_mutex_lock(&running_mutex);
 
     thread->running=2;
@@ -960,39 +975,26 @@ static void *filesort_fixed_heapsort_thread(thread_data *thread)
 
 static void *filesort_vary_heapsort_thread(thread_data *thread)
 {
- int fd;
  size_t item;
 
  /* Sort the data pointers using a heap sort */
 
  filesort_heapsort(thread->datap,thread->n,thread->compare);
 
- /* Create a temporary file and write the result */
+ /* Write the result to the given temporary file */
 
-#if defined(USE_PTHREADS) && USE_PTHREADS
+ if(thread->fd > 0)
+    for(item=0;item<thread->n;item++)
+      {
+       FILESORT_VARINT itemsize=*(FILESORT_VARINT*)((char*)thread->datap[item]-FILESORT_VARSIZE);
 
- if(option_filesort_threads>1)
-    pthread_mutex_lock(&files_mutex);
-
-#endif
-
- fd=OpenFileBufferedNew(thread->filename);
-
- for(item=0;item<thread->n;item++)
-   {
-    FILESORT_VARINT itemsize=*(FILESORT_VARINT*)((char*)thread->datap[item]-FILESORT_VARSIZE);
-
-    WriteFileBuffered(fd,(char*)thread->datap[item]-FILESORT_VARSIZE,itemsize+FILESORT_VARSIZE);
-   }
-
- CloseFileBuffered(fd);
+       WriteFileBuffered(thread->fd,(char*)thread->datap[item]-FILESORT_VARSIZE,itemsize+FILESORT_VARSIZE);
+      }
 
 #if defined(USE_PTHREADS) && USE_PTHREADS
 
  if(option_filesort_threads>1)
    {
-    pthread_mutex_unlock(&files_mutex);
-
     pthread_mutex_lock(&running_mutex);
 
     thread->running=2;
